//+------------------------------------------------------------------+
//|                                        AccountInfo Update v1.mq4 |
//|                        Copyright  2008-09, TrendLaboratory Ltd. |
//|            http://finance.groups.yahoo.com/group/TrendLaboratory |
//|                                   E-mail: igorad2003@yahoo.co.uk |
//+------------------------------------------------------------------+
#property copyright "Copyright  2008-09, TrendLaboratory"
#property link      "http://finance.groups.yahoo.com/group/TrendLaboratory"

#include <stderror.mqh>
#include <stdlib.mqh>

//---- input parameters
extern string     Expert_Name          = "AccountInfo Update";

extern string     AccountInfo          = "AccountInfo.txt"; // Name of account info File

extern string     StopTradeLossStr     = "StopTradeLoss";
extern double     StopTradeLossDiff    = 100.00;   // stop trading once it hits this amount less than the AccountBalance;

//////////

extern bool       UseTrailingProfit    = true;

extern string     TrailingProfitInfo   = "TrailingProfitInfo.txt"; // Name of tailing profit info File
extern string     TrailingProfitStr    = "TrailingProfit";

extern double     TrailingProfitStep   = 50;   // in dollars

//////////

extern bool       CloseAllTrades       = false;    // close trades based on reaching upper or lower targets
extern bool       CloseAllTradesUpper  = false;
extern double     CloseIfHigherThan    = 1000.00;
extern bool       CloseAllTradesLower  = false;
extern double     CloseIfLowerThan     = 1000.00;

//////////

extern int     AllowedSlippage        =  3;


double pipMultiplier = 1;

double StopTradeLoss = 0.0;   // stop trading once it hits this AccountBalance;


//+------------------------------------------------------------------+
//| expert initialization function                                   |
//+------------------------------------------------------------------+
int init()
{
//---- 

   CheckFilesExist();
        
   return(0);
}


//+------------------------------------------------------------------+
//| expert deinitialization function                                 |
//+------------------------------------------------------------------+
int deinit()
  {
//---- 
//----
   return(0);
  }
  
//+------------------------------------------------------------------+
//| expert start function                                            |
//+------------------------------------------------------------------+
int start()
{

   if (Digits==3 || Digits==5)
   {
      pipMultiplier = 10;
   }
   else  
   {
      pipMultiplier = 1;
   }

   UpdateAccountInfo();
   
   if(UseTrailingProfit)
   {
      CheckProfitTarget();
   }
   
   if(CloseAllTrades || CloseAllTradesUpper || CloseAllTradesLower)
   {
      CheckCloseTarget();
   }
   
//----
   return(0);
} // end start expert 
  
//+------------------------------------------------------------------+
//| UpdateAccountInfo function                                            |
//+------------------------------------------------------------------+
int UpdateAccountInfo()
{
   int handle;
   string tmpStopTradeLossStr;
   double StopTradeLoss;
   
   handle=FileOpen(AccountInfo,FILE_CSV|FILE_READ,';');
   
   if(handle<1)
   {
      Print("***Error - File not found ", AccountInfo, GetLastError());
      return(false);
   }
   else if(handle>=1)
   {
      if(!FileIsEnding(handle))
      {
         tmpStopTradeLossStr=FileReadString(handle);
      }
      else
      {
         Print("***Error - End of file reached: ", AccountInfo, GetLastError());
         return(false);     
      }
      
      if(!FileIsEnding(handle))
      {
         tmpStopTradeLossStr=FileReadString(handle);
      }
      else
      {
         Print("***Error - End of file reached: ", AccountInfo, GetLastError());
         return(false);     
      }
      
      StopTradeLoss = StrToDouble(tmpStopTradeLossStr);
      
      FileClose(handle);
   }
   
   if((StopTradeLoss+StopTradeLossDiff) < AccountBalance())
   // Update AccountInfo file as the AccountBalance() has increased
   {
      handle=FileOpen(AccountInfo,FILE_CSV|FILE_WRITE,';');
   
      if(handle<1)
      {
         Print("***Error - File not found ", GetLastError());
         return(false);
      }
      else if(handle>=1)
      {
         FileWrite(handle, StopTradeLossStr, DoubleToStr(AccountBalance() - StopTradeLossDiff,2));
         FileClose(handle);
         
         Comment(DoubleToStr(AccountBalance() - StopTradeLossDiff,2));
      }
   }
   
//----
   return(0);
} // end UpdateAccountInfo function

//+------------------------------------------------------------------+
//| CheckProfitTargetFileExists Function - added by nazmul           |
//+------------------------------------------------------------------+ 
bool CheckFilesExist()
{
   int handle;
   string tmpReadStr;
   double tmpProfit = 0.0;

   
   
   handle=FileOpen(AccountInfo,FILE_CSV|FILE_READ,';');

   if(handle<1)
   {
      Print("***Error - File not found - ", AccountInfo, " - ", GetLastError());
      handle=FileOpen(AccountInfo,FILE_CSV|FILE_READ|FILE_WRITE,';');
      if(handle<1)
      {
         Print("***Error - File can not be created - ", AccountInfo, " - ", GetLastError());
      }
      else
      {
         FileWrite(handle, StopTradeLossStr, "0.00");

         FileClose(handle);
      }
   }
   else
   {  // file exists
      FileClose(handle);
   }      


//////

   handle=FileOpen(TrailingProfitInfo,FILE_CSV|FILE_READ,';');

   if(handle<1)
   {
      Print("***Error - File not found - ", TrailingProfitInfo, " - ", GetLastError());
      handle=FileOpen(TrailingProfitInfo,FILE_CSV|FILE_READ|FILE_WRITE,';');
      if(handle<1)
      {
         Print("***Error - File can not be created - ", TrailingProfitInfo, " - ", GetLastError());
      }
      else
      {
         FileWrite(handle, TrailingProfitStr, "0.00");

         FileClose(handle);
      }
   }
   else
   {  // file exists
      FileClose(handle);
   }      

   return(false);
}

//+------------------------------------------------------------------+
//| CheckProfitTarget Function - added by nazmul                     |
//+------------------------------------------------------------------+ 
void CheckProfitTarget()
{
   int handle;
   string tmpReadStr;
   string CommentStr = "";
   double StopTradeLoss;
   double CurrentTrailingProfit;
   double tmpProfit = 0.0;
   

   if(AccountProfit() <= 0)
   { // profit target not reached
      return;
   }
   
   handle=FileOpen(TrailingProfitInfo,FILE_CSV|FILE_READ,';');

   if(handle<1)
   {
      Print("***Error - File not found ", TrailingProfitInfo, GetLastError());
   }
   else if(handle>=1)
   {
      if(!FileIsEnding(handle))
      {
         tmpReadStr=FileReadString(handle);
      }
      else
      {
         Print("***Error - End of file reached: ", TrailingProfitInfo, GetLastError());
         return(false);     
      }

      if(!FileIsEnding(handle))
      {
         tmpReadStr=FileReadString(handle);
      }
      else
      {
         Print("***Error - End of file reached: ", TrailingProfitInfo, GetLastError());
         return(false);     
      }

      CurrentTrailingProfit = StrToDouble(tmpReadStr);

      FileClose(handle); // close file read
   }
   
   // start processing trailing profit
   if(AccountProfit() <= CurrentTrailingProfit && CurrentTrailingProfit > 0.0)
   {
      // close all trades
      ExitAllTrades(Green, "Trailing profit hit");
      
      handle=FileOpen(TrailingProfitInfo,FILE_CSV|FILE_READ|FILE_WRITE,';');
      if(handle<1)
      {
         Print("***Error - File can not be created - ", TrailingProfitInfo, " - ", GetLastError());
   
         return(false);
      }
      else
      {
         FileWrite(handle, TrailingProfitStr, "0.00");

         FileClose(handle); // close file write
      }
      
      return;
   }
         
         
   if((AccountProfit() - TrailingProfitStep) > CurrentTrailingProfit)
   { // update profit file
      handle=FileOpen(TrailingProfitInfo,FILE_CSV|FILE_WRITE,';');
      if(handle<1)
      {
         Print("***Error - File can not be created - ", TrailingProfitInfo, " - ", GetLastError());

         return(false);
      }
      else
      {
         tmpProfit = MathFloor(AccountProfit()/TrailingProfitStep);
         if(tmpProfit < 2)
         {               
            FileWrite(handle, TrailingProfitStr, "0.00");

         }
         else
         {
            FileWrite(handle, TrailingProfitStr, DoubleToStr(((tmpProfit - 1.0) * TrailingProfitStep),2));
         }
      
         FileClose(handle);

         return;
      }
   }
   // end process trailing profit
   
//----
   return;
} // end CheckProfitTarget()

//+------------------------------------------------------------------+
//| CheckCloseTarget Function - added by nazmul                     |
//+------------------------------------------------------------------+ 
void CheckCloseTarget()
{

   if(CloseAllTrades)
   { // close all trades
      ExitAllTrades(Green, "Forced close all trades");
   }
   else
   {
      if(CloseAllTradesUpper && AccountProfit() >= CloseIfHigherThan)
      {
         ExitAllTrades(Green, "Forced close all trades");
      }
      if(CloseAllTradesLower && AccountProfit() <= CloseIfLowerThan)
      {
         ExitAllTrades(Green, "Forced close all trades");
      }
   }
   
   
//----
   return;
} // end CheckCloseTarget()

//+------------------------------------------------------------------+
//| Exit Trade Function - added by nazmul                            |
//+------------------------------------------------------------------+ 

void ExitAllTrades(color ColorValue, string ReasonStr)
{
   int Slippage = NormalizeDouble((999 * pipMultiplier),0);
   double cp;
   
   bool Success;
   int Cnt;
   
   for(Cnt=0;Cnt<OrdersTotal();Cnt++)
   {
      OrderSelect(Cnt,SELECT_BY_POS,MODE_TRADES);
      if(OrderType()==OP_BUY)
      {
         Success = OrderClose(OrderTicket(),OrderLots(),OrderClosePrice(),AllowedSlippage,Red);
         if(Success)
         {
            Print("*** " + ReasonStr + " - Closed = ", OrderSymbol());
         } 
         continue;
      }
      if(OrderType()==OP_SELL)
      {
         Success = OrderClose(OrderTicket(),OrderLots(),OrderClosePrice(),AllowedSlippage,Red);
         if(Success)
         {
            Print("*** " + ReasonStr + " - Closed = ", OrderSymbol());
         } 
         continue;
      }
      if(OrderType()==OP_BUYSTOP||OP_BUYLIMIT||OP_SELLSTOP||OP_SELLLIMIT)
      {
         OrderDelete(OrderTicket());
         continue;
      }
   } // end for
} // end ExitAllTrades


//+------------------------------------------------------------------+
//| ReadAccountInfo Function - added by nazmul                            |
//+------------------------------------------------------------------+ 
void ReadAccountInfo()
{    
   int handle;
   string StopTradeLossStr;
   
   handle=FileOpen(AccountInfo,FILE_CSV|FILE_READ,';');
   
   if(handle<1)
   {
      Print("***File not found ", GetLastError());
      return(false);
   }
   else if(handle>=1)
   {
      if(!FileIsEnding(handle))
      {
         StopTradeLossStr=FileReadString(handle);
      }
      else
      {
         return(0);
      }
      
      if(!FileIsEnding(handle))
      {
         StopTradeLossStr=FileReadString(handle);
      }
      else
      {
         return(0);
      }
      
      StopTradeLoss = StrToDouble(StopTradeLossStr);
      
      FileClose(handle);
   }
   
   return;
}